# -*- coding: utf-8 -*-
"""Test Setup of plone.mls.core."""

# python imports
import unittest2 as unittest

# zope imports
from plone.browserlayer import utils as layerutils

# local imports
from plone.mls.core.browser.interfaces import IMLSSpecific
from plone.mls.core.testing import PLONE_MLS_CORE_INTEGRATION_TESTING


class TestSetup(unittest.TestCase):
    """Setup Test Case for plone.mls.core."""
    layer = PLONE_MLS_CORE_INTEGRATION_TESTING

    def test_plone_app_registry_installed(self):
        """Test that plone.app.registry is installed."""
        portal = self.layer['portal']
        qi = portal.portal_quickinstaller
        if qi.isProductAvailable('plone.app.registry'):
            self.assertTrue(qi.isProductInstalled('plone.app.registry'))
        else:
            self.assertTrue(
                'plone.app.registry' in qi.listInstallableProfiles())

    def test_browserlayer_installed(self):
        """Test that the browser layer is installed correctly."""
        self.assertTrue(IMLSSpecific in layerutils.registered_layers())



# -*- coding: utf-8 -*-
"""Additional setup steps."""

# python imports
from logging import getLogger

# zope imports
from plone.browserlayer import utils as layerutils

# local imports
from plone.mls.core.browser.interfaces import IMLSSpecific

logger = getLogger('plone.mls.core')


def resetLayers(context):
    """Remove custom browser layer on uninstall."""

    if context.readDataFile('plone.mls.core_uninstall.txt') is None:
        return

    if IMLSSpecific in layerutils.registered_layers():
        layerutils.unregister_layer(name='plone.mls.core')
        logger.info('Browser layer "plone.mls.core" uninstalled.')



# -*- coding: utf-8 -*-
"""Test Layer for plone.mls.core."""

# zope imports
from plone.app.testing import (
    IntegrationTesting,
    PloneSandboxLayer,
    PLONE_FIXTURE,
    applyProfile,
)
from zope.configuration import xmlconfig


class PloneMLSCore(PloneSandboxLayer):
    """Custom Test Layer for plone.mls.core."""
    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        """Set up Zope for testing."""
        # Load ZCML
        import plone.mls.core
        xmlconfig.file('configure.zcml', plone.mls.core,
                       context=configurationContext)

    def setUpPloneSite(self, portal):
        """Set up a Plone site for testing."""
        applyProfile(portal, 'plone.mls.core:default')


PLONE_MLS_CORE_FIXTURE = PloneMLSCore()
PLONE_MLS_CORE_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_MLS_CORE_FIXTURE, ),
    name='PloneMLSCore:Integration',
)



# -*- coding: utf-8 -*-
"""Interface definitions."""

# zope imports
from zope import schema
from zope.interface import Interface

# local imports
from plone.mls.core.i18n import _


class IMLSSettings(Interface):
    """Global Propertyshelf MLS settings.

    This describes records stored in the configuration registry and obtainable
    via plone.registry.
    """

    mls_key = schema.TextLine(
        default=u"",
        required=False,
        title=_(
            u"label_mls_key",
            default=u"MLS API Key",
        )
    )

    mls_site = schema.TextLine(
        default=u"",
        required=True,
        title=_(
            u"label_mls_site",
            default=u"MLS URL",
        )
    )

    agency_id = schema.TextLine(
        default=u"",
        required=True,
        title=_(
            u"label_agency_id",
            default=u"Agency ID",
        )
    )



# -*- coding: utf-8 -*-
"""Install/Uninstall methods."""

# python imports
from logging import getLogger

# zope imports
from Products.CMFCore.utils import getToolByName


UNINSTALL_PROFILE = 'profile-plone.mls.core:uninstall'
logger = getLogger('plone.mls.core')


def uninstall(portal):
    portal_setup = getToolByName(portal, 'portal_setup')
    portal_setup.runAllImportStepsFromProfile(UNINSTALL_PROFILE)
    logger.info('Ran all uninstall steps.')



# -*- coding: utf-8 -*-
"""Interface definitions."""

# zope imports
from plone.theme.interfaces import IDefaultPloneLayer


class IMLSSpecific(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer."""



# -*- coding: utf-8 -*-
"""Package configuration."""

ERROR_404 = u'<p>The listing does not exist in the MLS.</p>'

ERROR_503 = u'<p>The MLS is currently not available. If the error persists, ' \
            u'please contact <a href="mailto:support@propertyshelf.com">' \
            u'support@propertyshelf.com</a>.</p><p>Please also double check ' \
            u'your <a href="%(portal_url)s/@@mls-controlpanel">MLS settings' \
            u'</a>.</p>'



# -*- coding: utf-8 -*-
"""Test Registry for plone.mls.core."""

# python imports
import unittest2 as unittest

# zope imports
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

# local imports
from plone.mls.core.interfaces import IMLSSettings
from plone.mls.core.testing import PLONE_MLS_CORE_INTEGRATION_TESTING


class TestMLSRegistry(unittest.TestCase):
    """Registry Test Case for plone.mls.core."""
    layer = PLONE_MLS_CORE_INTEGRATION_TESTING

    def test_registry_registered(self):
        """Test that the settings are registered correctly."""
        registry = getUtility(IRegistry)
        self.assertTrue(registry.forInterface(IMLSSettings))

    def test_mls_registry_agency_id(self):
        """Test for the 'agency_id' key and the default value."""
        registry = getUtility(IRegistry)
        key = 'plone.mls.core.interfaces.IMLSSettings.agency_id'
        self.assertTrue(key in registry.records.keys())
        self.assertEquals(registry.records.get(key).value, u'')

    def test_mls_registry_mls_key(self):
        """Test for the 'mls_key' key and the default value."""
        registry = getUtility(IRegistry)
        key = 'plone.mls.core.interfaces.IMLSSettings.mls_key'
        self.assertTrue(key in registry.records.keys())
        self.assertEquals(registry.records.get(key).value, u'')

    def test_mls_registry_mls_site(self):
        """Test for the 'mls_site' key and the default value."""
        registry = getUtility(IRegistry)
        key = 'plone.mls.core.interfaces.IMLSSettings.mls_site'
        self.assertTrue(key in registry.records.keys())
        self.assertEquals(registry.records.get(key).value, u'')



# -*- coding: utf-8 -*-
"""Test Control Panel for plone.mls.core."""

# python imports
import unittest2 as unittest

# zope imports
from Products.CMFCore.utils import getToolByName
from plone.app.testing import TEST_USER_ID, logout, setRoles
from plone.registry import Registry
from zope.component import getMultiAdapter
from zope.interface import directlyProvides

# local imports
from plone.mls.core.browser.interfaces import IMLSSpecific
from plone.mls.core.interfaces import IMLSSettings
from plone.mls.core.testing import PLONE_MLS_CORE_INTEGRATION_TESTING


class TestMLSControlPanel(unittest.TestCase):
    """Control Panel Test Case for plone.mls.core."""
    layer = PLONE_MLS_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Additional test setup."""
        self.portal = self.layer['portal']
        directlyProvides(self.portal.REQUEST, IMLSSpecific)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.registry = Registry()
        self.registry.registerInterface(IMLSSettings)

    def test_mls_controlpanel_view(self):
        """Test that the MLS configuration view is available."""
        view = getMultiAdapter((self.portal, self.portal.REQUEST),
                               name='mls-controlpanel')
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_mls_controlpanel_view_protected(self):
        """Test that the MLS configuration view needs authentication."""
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(Unauthorized, self.portal.restrictedTraverse,
                          '@@mls-controlpanel')

    def test_mls_in_controlpanel(self):
        """Check that there is an MLS entry in the control panel."""
        self.controlpanel = getToolByName(self.portal, 'portal_controlpanel')
        self.assertTrue('propertyshelf_mls' in [a.getAction(self)['id']
            for a in self.controlpanel.listActions()])



# -*- coding: utf-8 -*-
"""Setup for plone.mls.core package."""

import os
from setuptools import setup, find_packages

__version__ = '0.4.1'


setup(
    name='plone.mls.core',
    version=__version__,
    description="Plone support for the Propertyshelf MLS.",
    long_description='\n\n'.join([
        open("README.txt").read() + "\n" +
        open(os.path.join("docs", "HISTORY.txt")).read(),
    ]),
    # Get more strings from
    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Intended Audience :: Other Audience",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Zope",
    ],
    keywords='plone zope mls listing',
    author='Propertyshelf, Inc.',
    author_email='development@propertyshelf.com',
    url='https://bitbucket.org/propertyshelf/plone.mls.core',
    download_url='http://pypi.python.org/pypi/plone.mls.core',
    license='GPL',
    packages=find_packages('src', exclude=['ez_setup']),
    package_dir={'': 'src'},
    namespace_packages=['plone', 'plone.mls'],
    include_package_data=True,
    zip_safe=False,
    extras_require={
        'test': [
            'plone.app.testing',
        ],
    },
    install_requires=[
        'setuptools',
        'Plone',
        'plone.app.registry',
        'httplib2',
        'simplejson',
    ],
    entry_points="""
    # -*- Entry points: -*-

    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
"""Propertyshelf MLS utils."""

# python imports
from urllib import urlencode
import errno
import httplib2
import simplejson
import socket
import urllib2
# import json

# zope imports
from Acquisition import aq_inner, aq_parent
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

# local imports
from plone.mls.core.interfaces import IMLSSettings


TIMEOUT = 10


class MLSError:
    """General MLS Error."""
    def __init__(self, code=None, reason=None):
        self.code = code
        self.reason = reason


class MLSConnectionError(MLSError):
    """No Conncetion to the MLS."""


class MLSDataError(MLSError):
    """Data returned from the MLS contains errors."""


def authenticate():
    """Authenticate the Plone Website."""
    registry = getUtility(IRegistry)
    settings = registry.forInterface(IMLSSettings)
#    test_json()

    if settings.mls_site is None and settings.agency_id is None:
        return False

    url = '%(site)s/agencies/%(agency_id)s' % dict(
        site=settings.mls_site,
        agency_id=settings.agency_id,
    )
    url = urllib2.unquote(url)

    try:
        urllib2.urlopen(url)
    except IOError:
        return False

    return True


def get_language(context):
    portal_state = context.unrestrictedTraverse("@@plone_portal_state")
    return aq_inner(context).Language() or \
           aq_inner(aq_parent(context)).Language() or \
           portal_state.default_language()


def get_listing(lid, summary=False, lang=None):
    """Get the data for a single listing."""
    kwargs = {}
    registry = getUtility(IRegistry)
    settings = registry.forInterface(IMLSSettings)

    if settings.mls_site is None or len(settings.mls_site) == 0:
        raise MLSConnectionError(code=503)

    if settings.agency_id is None or len(settings.agency_id) == 0:
        raise MLSConnectionError(code=503)

    URL_BASE = '%(site)s/api/listings/listing/%(listing_id)s/agency/%' \
               '(agency_id)s' % dict(
        site=settings.mls_site,
        agency_id=settings.agency_id,
        listing_id=lid,
    )

    kwargs.update({
        'apikey': settings.mls_key,
    })

    if lang:
        kwargs.update({'lang': lang})

    if summary:
        kwargs.update({'summary': 1})

    url = URL_BASE + '?' + urlencode(kwargs)
    h = httplib2.Http(".cache")
    try:
        resp, content = h.request(url)
    except socket.error, e:
        err = 0
        if hasattr(e, 'args'):
            err = getattr(e, 'args')[0]
        else:
            err = e.errno
        if err == errno.ECONNREFUSED:  # Connection refused
            raise MLSConnectionError
    try:
        result = simplejson.loads(content)
    except simplejson.JSONDecodeError, e:
        raise MLSDataError(code=401)

    if result.get('status', 'error') != 'ok':
        raise MLSDataError
    return result.get('result', None)



# -*- coding: utf-8 -*-
"""Custom Batch Provider for listing results."""

# zope imports
from Products.CMFPlone.PloneBatch import Batch


class ListingBatch(Batch):
    """Listing result batch."""

    def __init__(self, sequence, size, start=0, end=0, orphan=0, overlap=0,
                 pagerange=7, quantumleap=0, b_start_str='b_start',
                 batch_data=None):
        self.batch_data = batch_data

        if sequence is None:
            sequence = []

        super(ListingBatch, self).__init__(
            sequence, size, start, end, orphan, overlap, pagerange,
            quantumleap, b_start_str,
        )

    @property
    def sequence_length(self):
        """Effective length of sequence."""
        if self.batch_data is not None:
            length = getattr(self, 'pagesize', 0)
            return self.batch_data.get('results', length)
        return super(ListingBatch, self).sequence_length

    def __getitem__(self, index):
        if index >= self.length:
            raise IndexError(index)
        return self._sequence[index]

    @property
    def next(self):
        """Next batch page."""
        if self.end >= (self.last + self.pagesize):
            return None
        return ListingBatch(
            self._sequence, self._size, self.end - self.overlap, 0,
            self.orphan, self.overlap, batch_data=self.batch_data,
        )

    @property
    def previous(self):
        """Previous batch page."""
        if not self.first:
            return None
        return ListingBatch(
            self._sequence, self._size, self.first - self._size + self.overlap,
            0, self.orphan, self.overlap, batch_data=self.batch_data,
        )



# -*- coding: utf-8 -*-
"""MLS Settings Control Panel."""

# zope imports
from plone.app.registry.browser import controlpanel

# local imports
from plone.mls.core.i18n import _
from plone.mls.core.interfaces import IMLSSettings


class MLSSettingsEditForm(controlpanel.RegistryEditForm):
    """MLS Settings Form"""

    schema = IMLSSettings
    label = _(u"heading_mls_settings", u"Propertyshelf MLS Settings")

    def updateFields(self):
        super(MLSSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(MLSSettingsEditForm, self).updateWidgets()


class MLSSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    """MLS Settings Control Panel"""

    form = MLSSettingsEditForm
